@@ -2,7 +2,6 @@
 
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RecordWildCards #-}
-{-# LANGUAGE CPP #-}
 
 module Hledger.UI.AccountsScreen
  (accountsScreen
@@ -13,23 +12,20 @@ where
 
 import Brick
 import Brick.Widgets.List
-  (handleListEvent, list, listElementsL, listMoveDown, listMoveTo, listNameL, listSelectedElement, listSelectedL, renderList)
 import Brick.Widgets.Edit
 import Control.Monad
 import Control.Monad.IO.Class (liftIO)
-import Data.List
+import Data.List hiding (reverse)
 import Data.Maybe
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid ((<>))
-#endif
 import qualified Data.Text as T
-import Data.Time.Calendar (Day, addDays)
+import Data.Time.Calendar (Day)
 import qualified Data.Vector as V
-import Graphics.Vty (Event(..),Key(..),Modifier(..))
+import Graphics.Vty (Event(..),Key(..),Modifier(..), Button (BLeft, BScrollDown, BScrollUp))
 import Lens.Micro.Platform
 import Safe
 import System.Console.ANSI
 import System.FilePath (takeFileName)
+import Text.DocLayout (realLength)
 
 import Hledger
 import Hledger.Cli hiding (progname,prognameandversion)
@@ -40,6 +36,7 @@ import Hledger.UI.UIUtils
 import Hledger.UI.Editor
 import Hledger.UI.RegisterScreen
 import Hledger.UI.ErrorScreen
+import Data.Vector ((!?))
 
 
 accountsScreen :: Screen
@@ -53,11 +50,11 @@ accountsScreen = AccountsScreen{
 
 asInit :: Day -> Bool -> UIState -> UIState
 asInit d reset ui@UIState{
-  aopts=uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}},
+  aopts=UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec@ReportSpec{_rsReportOpts=ropts}}},
   ajournal=j,
   aScreen=s@AccountsScreen{}
   } =
-  ui{aopts=uopts', aScreen=s & asList .~ newitems'}
+  ui{aScreen=s & asList .~ newitems'}
    where
     newitems = list AccountsList (V.fromList $ displayitems ++ blankitems) 1
 
@@ -73,61 +70,52 @@ asInit d reset ui@UIState{
                    (_, Nothing)            -> 0
                    (_, Just (_,AccountsScreenItem{asItemAccountName=a})) ->
                      headDef 0 $ catMaybes [
-                       findIndex (a ==) as
+                       elemIndex a as
                       ,findIndex (a `isAccountNamePrefixOf`) as
                       ,Just $ max 0 (length (filter (< a) as) - 1)
                       ]
                       where
                         as = map asItemAccountName displayitems
 
-    uopts' = uopts{cliopts_=copts{reportopts_=ropts'}}
-    ropts' = ropts{accountlistmode_=if tree_ ropts then ALTree else ALFlat}
-
-    q = And [queryFromOpts d ropts, excludeforecastq (forecast_ ropts)]
-      where
-        -- Except in forecast mode, exclude future/forecast transactions.
-        excludeforecastq (Just _) = Any
-        excludeforecastq Nothing  =  -- not:date:tomorrow- not:tag:generated-transaction
-          And [
-             Not (Date $ DateSpan (Just $ addDays 1 d) Nothing)
-            ,Not generatedTransactionTag
-          ]
+    rspec' =
+      -- Further restrict the query based on the current period and future/forecast mode.
+      (reportSpecSetFutureAndForecast d (forecast_ $ inputopts_ copts) rspec)
+      -- always show declared accounts even if unused
+        {_rsReportOpts=ropts{declared_=True}}
 
     -- run the report
-    (items,_total) = balanceReport ropts' q j
+    (items,_total) = balanceReport rspec' j
 
     -- pre-render the list items
     displayitem (fullacct, shortacct, indent, bal) =
       AccountsScreenItem{asItemIndentLevel        = indent
                         ,asItemAccountName        = fullacct
-                        ,asItemDisplayAccountName = replaceHiddenAccountsNameWith "All" $ if tree_ ropts' then shortacct else fullacct
-                        ,asItemRenderedAmounts    = map (showAmountWithoutPrice False) amts
+                        ,asItemDisplayAccountName = replaceHiddenAccountsNameWith "All" $ if tree_ ropts then shortacct else fullacct
+                        ,asItemMixedAmount        = Just bal
                         }
-      where
-        Mixed amts = normaliseMixedAmountSquashPricesForDisplay $ stripPrices bal
-        stripPrices (Mixed as) = Mixed $ map stripprice as where stripprice a = a{aprice=Nothing}
     displayitems = map displayitem items
-    -- blanks added for scrolling control, cf RegisterScreen
+    -- blanks added for scrolling control, cf RegisterScreen.
+    -- XXX Ugly. Changing to 0 helps when debugging.
     blankitems = replicate 100
       AccountsScreenItem{asItemIndentLevel        = 0
                         ,asItemAccountName        = ""
                         ,asItemDisplayAccountName = ""
-                        ,asItemRenderedAmounts    = []
+                        ,asItemMixedAmount        = Nothing
                         }
 
 
 asInit _ _ _ = error "init function called with wrong screen type, should not happen"  -- PARTIAL:
 
 asDraw :: UIState -> [Widget Name]
-asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
+asDraw UIState{aopts=_uopts@UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec}}
               ,ajournal=j
               ,aScreen=s@AccountsScreen{}
               ,aMode=mode
               } =
-  case mode of
-    Help       -> [helpDialog copts, maincontent]
-    -- Minibuffer e -> [minibuffer e, maincontent]
-    _          -> [maincontent]
+    case mode of
+      Help       -> [helpDialog copts, maincontent]
+      -- Minibuffer e -> [minibuffer e, maincontent]
+      _          -> [maincontent]
   where
     maincontent = Widget Greedy Greedy $ do
       c <- getContext
@@ -137,47 +125,40 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
           c^.availWidthL
           - 2 -- XXX due to margin ? shouldn't be necessary (cf UIUtils)
         displayitems = s ^. asList . listElementsL
-        maxacctwidthseen =
-          -- ltrace "maxacctwidthseen" $
-          V.maximum $
-          V.map (\AccountsScreenItem{..} -> asItemIndentLevel + Hledger.Cli.textWidth asItemDisplayAccountName) $
-          -- V.filter (\(indent,_,_,_) -> (indent-1) <= fromMaybe 99999 mdepth) $
-          displayitems
-        maxbalwidthseen =
-          -- ltrace "maxbalwidthseen" $
-          V.maximum $ V.map (\AccountsScreenItem{..} -> sum (map strWidth asItemRenderedAmounts) + 2 * (length asItemRenderedAmounts - 1)) displayitems
-        maxbalwidth =
-          -- ltrace "maxbalwidth" $
-          max 0 (availwidth - 2 - 4) -- leave 2 whitespace plus least 4 for accts
-        balwidth =
-          -- ltrace "balwidth" $
-          min maxbalwidth maxbalwidthseen
-        maxacctwidth =
-          -- ltrace "maxacctwidth" $
-          availwidth - 2 - balwidth
-        acctwidth =
-          -- ltrace "acctwidth" $
-          min maxacctwidth maxacctwidthseen
 
-        -- XXX how to minimise the balance column's jumping around
-        -- as you change the depth limit ?
+        acctwidths = V.map (\AccountsScreenItem{..} -> asItemIndentLevel + realLength asItemDisplayAccountName) displayitems
+        balwidths  = V.map (maybe 0 (wbWidth . showMixedAmountB oneLine) . asItemMixedAmount) displayitems
+        preferredacctwidth = V.maximum acctwidths
+        totalacctwidthseen = V.sum acctwidths
+        preferredbalwidth  = V.maximum balwidths
+        totalbalwidthseen  = V.sum balwidths
 
-        colwidths = (acctwidth, balwidth)
+        totalwidthseen = totalacctwidthseen + totalbalwidthseen
+        shortfall = preferredacctwidth + preferredbalwidth + 2 - availwidth
+        acctwidthproportion = fromIntegral totalacctwidthseen / fromIntegral totalwidthseen
+        adjustedacctwidth = min preferredacctwidth . max 15 . round $ acctwidthproportion * fromIntegral (availwidth - 2)  -- leave 2 whitespace for padding
+        adjustedbalwidth  = availwidth - 2 - adjustedacctwidth
+
+        -- XXX how to minimise the balance column's jumping around as you change the depth limit ?
+
+        colwidths | shortfall <= 0 = (preferredacctwidth, preferredbalwidth)
+                  | otherwise      = (adjustedacctwidth, adjustedbalwidth)
 
       render $ defaultLayout toplabel bottomlabel $ renderList (asDrawItem colwidths) True (_asList s)
 
       where
-        ishistorical = balancetype_ ropts == HistoricalBalance
+        ropts = _rsReportOpts rspec
+        ishistorical = balanceaccum_ ropts == Historical
 
         toplabel =
               withAttr ("border" <> "filename") files
           <+> toggles
           <+> str (" account " ++ if ishistorical then "balances" else "changes")
           <+> borderPeriodStr (if ishistorical then "at end of" else "in") (period_ ropts)
-          <+> borderQueryStr querystr
+          <+> borderQueryStr (T.unpack . T.unwords . map textQuoteIfNeeded $ querystring_ ropts)
           <+> borderDepthStr mdepth
           <+> str (" ("++curidx++"/"++totidx++")")
-          <+> (if ignore_assertions_ $ inputopts_ copts
+          <+> (if ignore_assertions_ . balancingopts_ $ inputopts_ copts
                then withAttr ("border" <> "query") (str " ignoring balance assertions")
                else str "")
           where
@@ -192,7 +173,6 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
               ,uiShowStatus copts $ statuses_ ropts
               ,if real_ ropts then ["real"] else []
               ]
-            querystr = query_ ropts
             mdepth = depth_ ropts
             curidx = case _asList s ^. listSelectedL of
                        Nothing -> "-"
@@ -202,8 +182,8 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
                 nonblanks = V.takeWhile (not . T.null . asItemAccountName) $ s ^. asList . listElementsL
 
         bottomlabel = case mode of
-                        Minibuffer ed -> minibuffer ed
-                        _             -> quickhelp
+                        Minibuffer label ed -> minibuffer label ed
+                        _                   -> quickhelp
           where
             quickhelp = borderKeysStr' [
                ("?", str "help")
@@ -213,7 +193,7 @@ asDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
               -- ,("l", str "list")
               ,("-+", str "depth")
               ,("H", renderToggle (not ishistorical) "end-bals" "changes")
-              ,("F", renderToggle1 (isJust $ forecast_ ropts) "forecast")
+              ,("F", renderToggle1 (isJust . forecast_ $ inputopts_ copts) "forecast")
               --,("/", "filter")
               --,("DEL", "unfilter")
               --,("ESC", "cancel/top")
@@ -230,36 +210,29 @@ asDrawItem (acctwidth, balwidth) selected AccountsScreenItem{..} =
     -- c <- getContext
       -- let showitem = intercalate "\n" . balanceReportItemAsText defreportopts fmt
     render $
-      addamts asItemRenderedAmounts $
-      str (T.unpack $ fitText (Just acctwidth) (Just acctwidth) True True $ T.replicate (asItemIndentLevel) " " <> asItemDisplayAccountName) <+>
-      str "  " <+>
-      str (balspace asItemRenderedAmounts)
+      txt (fitText (Just acctwidth) (Just acctwidth) True True $ T.replicate (asItemIndentLevel) " " <> asItemDisplayAccountName) <+>
+      txt balspace <+>
+      splitAmounts balBuilder
       where
-        balspace as = replicate n ' '
-          where n = max 0 (balwidth - (sum (map strWidth as) + 2 * (length as - 1)))
-        addamts :: [String] -> Widget Name -> Widget Name
-        addamts [] w = w
-        addamts [a] w = (<+> renderamt a) w
-        -- foldl' :: (b -> a -> b) -> b -> t a -> b
-        -- foldl' (Widget -> String -> Widget) -> Widget -> [String] -> Widget
-        addamts (a:as) w = foldl' addamt (addamts [a] w) as
-        addamt :: Widget Name -> String -> Widget Name
-        addamt w a = ((<+> renderamt a) . (<+> str ", ")) w
-        renderamt :: String -> Widget Name
-        renderamt a | '-' `elem` a = withAttr (sel $ "list" <> "balance" <> "negative") $ str a
-                    | otherwise    = withAttr (sel $ "list" <> "balance" <> "positive") $ str a
+        balBuilder = maybe mempty showamt asItemMixedAmount
+        showamt = showMixedAmountB oneLine{displayMinWidth=Just balwidth, displayMaxWidth=Just balwidth}
+        balspace = T.replicate (2 + balwidth - wbWidth balBuilder) " "
+        splitAmounts = foldr1 (<+>) . intersperse (str ", ") . map renderamt . T.splitOn ", " . wbToText
+        renderamt :: T.Text -> Widget Name
+        renderamt a | T.any (=='-') a = withAttr (sel $ "list" <> "balance" <> "negative") $ txt a
+                    | otherwise       = withAttr (sel $ "list" <> "balance" <> "positive") $ txt a
         sel | selected  = (<> "selected")
             | otherwise = id
 
 asHandle :: UIState -> BrickEvent Name AppEvent -> EventM Name (Next UIState)
 asHandle ui0@UIState{
    aScreen=scr@AccountsScreen{..}
-  ,aopts=UIOpts{cliopts_=copts}
+  ,aopts=UIOpts{uoCliOpts=copts}
   ,ajournal=j
   ,aMode=mode
   } ev = do
-  d <- liftIO getCurrentDay
   let
+    d = copts^.rsDay
     nonblanks = V.takeWhile (not . T.null . asItemAccountName) $ _asList^.listElementsL
     lastnonblankidx = max 0 (length nonblanks - 1)
 
@@ -271,18 +244,21 @@ asHandle ui0@UIState{
     ui = ui0{aScreen=scr & asSelectedAccount .~ selacct}
 
   case mode of
-    Minibuffer ed ->
+    Minibuffer _ ed ->
       case ev of
         VtyEvent (EvKey KEsc   []) -> continue $ closeMinibuffer ui
-        VtyEvent (EvKey KEnter []) -> continue $ regenerateScreens j d $ setFilter s $ closeMinibuffer ui
-                            where s = chomp $ unlines $ map strip $ getEditContents ed
+        VtyEvent (EvKey KEnter []) -> continue $ regenerateScreens j d $
+            case setFilter s $ closeMinibuffer ui of
+              Left bad -> showMinibuffer "Cannot compile regular expression" (Just bad) ui
+              Right ui' -> ui'
+          where s = chomp $ unlines $ map strip $ getEditContents ed
         VtyEvent (EvKey (KChar 'l') [MCtrl]) -> redraw ui
         VtyEvent (EvKey (KChar 'z') [MCtrl]) -> suspend ui
         VtyEvent ev        -> do ed' <- handleEditorEvent ev ed
-                                 continue $ ui{aMode=Minibuffer ed'}
+                                 continue $ ui{aMode=Minibuffer "filter" ed'}
         AppEvent _        -> continue ui
-        MouseDown _ _ _ _ -> continue ui
-        MouseUp _ _ _     -> continue ui
+        MouseDown{}       -> continue ui
+        MouseUp{}         -> continue ui
 
     Help ->
       case ev of
@@ -296,7 +272,7 @@ asHandle ui0@UIState{
         VtyEvent (EvKey (KChar 'q') []) -> halt ui
         -- EvKey (KChar 'l') [MCtrl] -> do
         VtyEvent (EvKey KEsc        []) -> continue $ resetScreens d ui
-        VtyEvent (EvKey (KChar c)   []) | c `elem` ['?'] -> continue $ setMode Help ui
+        VtyEvent (EvKey (KChar c)   []) | c == '?' -> continue $ setMode Help ui
         -- XXX AppEvents currently handled only in Normal mode
         -- XXX be sure we don't leave unconsumed events piling up
         AppEvent (DateChange old _) | isStandardPeriod p && p `periodContainsDate` old ->
@@ -309,7 +285,7 @@ asHandle ui0@UIState{
         VtyEvent (EvKey (KChar 'a') []) -> suspendAndResume $ clearScreen >> setCursorPosition 0 0 >> add copts j >> uiReloadJournalIfChanged copts d j ui
         VtyEvent (EvKey (KChar 'A') []) -> suspendAndResume $ void (runIadd (journalFilePath j)) >> uiReloadJournalIfChanged copts d j ui
         VtyEvent (EvKey (KChar 'E') []) -> suspendAndResume $ void (runEditor endPosition (journalFilePath j)) >> uiReloadJournalIfChanged copts d j ui
-        VtyEvent (EvKey (KChar 'B') []) -> continue $ regenerateScreens j d $ toggleCost ui
+        VtyEvent (EvKey (KChar 'B') []) -> continue $ regenerateScreens j d $ toggleConversionOp ui
         VtyEvent (EvKey (KChar 'V') []) -> continue $ regenerateScreens j d $ toggleValue ui
         VtyEvent (EvKey (KChar '0') []) -> continue $ regenerateScreens j d $ setDepth (Just 0) ui
         VtyEvent (EvKey (KChar '1') []) -> continue $ regenerateScreens j d $ setDepth (Just 1) ui
@@ -329,7 +305,7 @@ asHandle ui0@UIState{
         -- display mode/query toggles
         VtyEvent (EvKey (KChar 'H') []) -> asCenterAndContinue $ regenerateScreens j d $ toggleHistorical ui
         VtyEvent (EvKey (KChar 't') []) -> asCenterAndContinue $ regenerateScreens j d $ toggleTree ui
-        VtyEvent (EvKey (KChar 'Z') []) -> asCenterAndContinue $ regenerateScreens j d $ toggleEmpty ui
+        VtyEvent (EvKey (KChar c) []) | c `elem` ['z','Z'] -> asCenterAndContinue $ regenerateScreens j d $ toggleEmpty ui
         VtyEvent (EvKey (KChar 'R') []) -> asCenterAndContinue $ regenerateScreens j d $ toggleReal ui
         VtyEvent (EvKey (KChar 'U') []) -> asCenterAndContinue $ regenerateScreens j d $ toggleUnmarked ui
         VtyEvent (EvKey (KChar 'P') []) -> asCenterAndContinue $ regenerateScreens j d $ togglePending ui
@@ -340,7 +316,7 @@ asHandle ui0@UIState{
         VtyEvent (EvKey (KUp)       [MShift]) -> continue $ regenerateScreens j d $ growReportPeriod d ui
         VtyEvent (EvKey (KRight)    [MShift]) -> continue $ regenerateScreens j d $ nextReportPeriod journalspan ui
         VtyEvent (EvKey (KLeft)     [MShift]) -> continue $ regenerateScreens j d $ previousReportPeriod journalspan ui
-        VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer ui
+        VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer "filter" Nothing ui
         VtyEvent (EvKey k           []) | k `elem` [KBS, KDel] -> (continue $ regenerateScreens j d $ resetFilter ui)
         VtyEvent e | e `elem` moveLeftEvents -> continue $ popScreen ui
         VtyEvent (EvKey (KChar 'l') [MCtrl]) -> scrollSelectionToMiddle _asList >> redraw ui
@@ -349,24 +325,29 @@ asHandle ui0@UIState{
         -- enter register screen for selected account (if there is one),
         -- centering its selected transaction if possible
         VtyEvent e | e `elem` moveRightEvents
-                   , not $ isBlankElement $ listSelectedElement _asList->
-          -- TODO center selection after entering register screen; neither of these works till second time entering; easy strictifications didn't help
-          rsCenterAndContinue $
-          -- flip rsHandle (VtyEvent (EvKey (KChar 'l') [MCtrl])) $
-            screenEnter d regscr ui
-          where
-            regscr = rsSetAccount selacct isdepthclipped registerScreen
-            isdepthclipped = case getDepth ui of
-                                Just d  -> accountNameLevel selacct >= d
-                                Nothing -> False
+                   , not $ isBlankElement $ listSelectedElement _asList -> asEnterRegister d selacct ui
 
-        -- prevent moving down over blank padding items;
-        -- instead scroll down by one, until maximally scrolled - shows the end has been reached
-        VtyEvent (EvKey (KDown)     []) | isBlankElement mnextelement -> do
-          vScrollBy (viewportScroll $ _asList^.listNameL) 1
-          continue ui
-          where
-            mnextelement = listSelectedElement $ listMoveDown _asList
+        -- MouseDown is sometimes duplicated, https://github.com/jtdaugherty/brick/issues/347
+        -- just use it to move the selection
+        MouseDown _n BLeft _mods Location{loc=(_x,y)} | not $ (=="") clickedacct -> do
+          continue ui{aScreen=scr{_asList=listMoveTo y _asList}}
+          where clickedacct = maybe "" asItemAccountName $ listElements _asList !? y
+        -- and on MouseUp, enter the subscreen
+        MouseUp _n (Just BLeft) Location{loc=(_x,y)} | not $ (=="") clickedacct -> do
+          asEnterRegister d clickedacct ui
+          where clickedacct = maybe "" asItemAccountName $ listElements _asList !? y
+
+        -- when selection is at the last item, DOWN scrolls instead of moving, until maximally scrolled
+        VtyEvent e | e `elem` moveDownEvents, isBlankElement mnextelement -> do
+          vScrollBy (viewportScroll $ _asList^.listNameL) 1 >> continue ui
+          where mnextelement = listSelectedElement $ listMoveDown _asList
+
+        -- mouse scroll wheel scrolls the viewport up or down to its maximum extent,
+        -- pushing the selection when necessary.
+        MouseDown name btn _mods _loc | btn `elem` [BScrollUp, BScrollDown] -> do
+          let scrollamt = if btn==BScrollUp then -1 else 1
+          list' <- listScrollPushingSelection name _asList (asListSize _asList) scrollamt
+          continue ui{aScreen=scr{_asList=list'}}
 
         -- if page down or end leads to a blank padding item, stop at last non-blank
         VtyEvent e@(EvKey k           []) | k `elem` [KPageDown, KEnd] -> do
@@ -386,15 +367,25 @@ asHandle ui0@UIState{
                                     & asSelectedAccount .~ selacct
                                     }
 
-        AppEvent _        -> continue ui
-        MouseDown _ _ _ _ -> continue ui
-        MouseUp _ _ _     -> continue ui
+        MouseDown{} -> continue ui
+        MouseUp{}   -> continue ui
+        AppEvent _  -> continue ui
 
   where
     journalspan = journalDateSpan False j
 
 asHandle _ _ = error "event handler called with wrong screen type, should not happen"  -- PARTIAL:
 
+asEnterRegister d selacct ui = do
+  rsCenterAndContinue $
+  -- flip rsHandle (VtyEvent (EvKey (KChar 'l') [MCtrl])) $
+    screenEnter d regscr ui
+      where
+        regscr = rsSetAccount selacct isdepthclipped registerScreen
+        isdepthclipped = case getDepth ui of
+                            Just d  -> accountNameLevel selacct >= d
+                            Nothing -> False
+
 asSetSelectedAccount a s@AccountsScreen{} = s & asSelectedAccount .~ a
 asSetSelectedAccount _ s = s
 
@@ -403,3 +394,5 @@ isBlankElement mel = ((asItemAccountName . snd) <$> mel) == Just ""
 asCenterAndContinue ui = do
   scrollSelectionToMiddle $ _asList $ aScreen ui
   continue ui
+
+asListSize = V.length . V.takeWhile ((/="").asItemAccountName) . listElements
