@@ -1,7 +1,8 @@
 -- The error screen, showing a current error condition (such as a parse error after reloading the journal)
 
-{-# LANGUAGE OverloadedStrings, FlexibleContexts, RecordWildCards #-}
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleContexts  #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards   #-}
 
 module Hledger.UI.ErrorScreen
  (errorScreen
@@ -15,12 +16,10 @@ import Brick
 -- import Brick.Widgets.Border ("border")
 import Control.Monad
 import Control.Monad.IO.Class (liftIO)
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid
-#endif
 import Data.Time.Calendar (Day)
 import Data.Void (Void)
 import Graphics.Vty (Event(..),Key(..),Modifier(..))
+import Lens.Micro ((^.))
 import Text.Megaparsec
 import Text.Megaparsec.Char
 
@@ -44,7 +43,7 @@ esInit _ _ ui@UIState{aScreen=ErrorScreen{}} = ui
 esInit _ _ _ = error "init function called with wrong screen type, should not happen"  -- PARTIAL:
 
 esDraw :: UIState -> [Widget Name]
-esDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{}}
+esDraw UIState{aopts=UIOpts{uoCliOpts=copts}
               ,aScreen=ErrorScreen{..}
               ,aMode=mode
               } =
@@ -60,9 +59,10 @@ esDraw UIState{aopts=UIOpts{cliopts_=copts@CliOpts{}}
               withAttr ("border" <> "bold") (str "Oops. Please fix this problem then press g to reload")
               -- <+> (if ignore_assertions_ copts then withAttr ("border" <> "query") (str " ignoring") else str " not ignoring")
 
-        bottomlabel = case mode of
+        bottomlabel = quickhelp
+                        -- case mode of
                         -- Minibuffer ed -> minibuffer ed
-                        _             -> quickhelp
+                        -- _             -> quickhelp
           where
             quickhelp = borderKeysStr [
                ("h", "help")
@@ -76,7 +76,7 @@ esDraw _ = error "draw function called with wrong screen type, should not happen
 
 esHandle :: UIState -> BrickEvent Name AppEvent -> EventM Name (Next UIState)
 esHandle ui@UIState{aScreen=ErrorScreen{..}
-                   ,aopts=UIOpts{cliopts_=copts}
+                   ,aopts=UIOpts{uoCliOpts=copts}
                    ,ajournal=j
                    ,aMode=mode
                    }
@@ -90,7 +90,7 @@ esHandle ui@UIState{aScreen=ErrorScreen{..}
         _                    -> helpHandle ui ev
 
     _ -> do
-      d <- liftIO getCurrentDay
+      let d = copts^.rsDay
       case ev of
         VtyEvent (EvKey (KChar 'q') []) -> halt ui
         VtyEvent (EvKey KEsc        []) -> continue $ uiCheckBalanceAssertions d $ resetScreens d ui
@@ -137,32 +137,42 @@ hledgerparseerrorpositionp = do
       ]
 
 
--- Unconditionally reload the journal, regenerating the current screen
--- and all previous screens in the history.
+-- | Unconditionally reload the journal, regenerating the current screen
+-- and all previous screens in the history as of the provided today-date.
 -- If reloading fails, enter the error screen, or if we're already
 -- on the error screen, update the error displayed.
--- The provided CliOpts are used for reloading, and then saved
--- in the UIState if reloading is successful (otherwise the
--- ui state keeps its old cli opts.)
 -- Defined here so it can reference the error screen.
+--
+-- The provided CliOpts are used for reloading, and then saved in the
+-- UIState if reloading is successful (otherwise the UIState keeps its old
+-- CliOpts.) (XXX needed for.. ?)
+--
+-- Forecasted transactions are always generated, as at hledger-ui startup.
+-- If a forecast period is specified in the provided opts, or was specified
+-- at startup, it is preserved.
+--
 uiReloadJournal :: CliOpts -> Day -> UIState -> IO UIState
 uiReloadJournal copts d ui = do
-  ej <- journalReload copts
+  ej <-
+    let copts' = enableForecastPreservingPeriod ui copts
+    in journalReload copts'
   return $ case ej of
-    Right j  -> regenerateScreens j d ui{aopts=(aopts ui){cliopts_=copts}}
+    Right j  -> regenerateScreens j d ui
     Left err ->
       case ui of
         UIState{aScreen=s@ErrorScreen{}} -> ui{aScreen=s{esError=err}}
         _                                -> screenEnter d errorScreen{esError=err} ui
 
--- Like uiReloadJournal, but does not bother re-parsing the journal if
--- the file(s) have not changed since last loaded. Always regenerates
--- the current and previous screens though, since opts or date may have changed.
+-- | Like uiReloadJournal, but does not re-parse the journal if the file(s)
+-- have not changed since last loaded. Always regenerates the screens though,
+-- since the provided options or today-date may have changed.
 uiReloadJournalIfChanged :: CliOpts -> Day -> Journal -> UIState -> IO UIState
 uiReloadJournalIfChanged copts d j ui = do
-  (ej, _changed) <- journalReloadIfChanged copts d j
+  (ej, _changed) <-
+    let copts' = enableForecastPreservingPeriod ui copts
+    in journalReloadIfChanged copts' d j
   return $ case ej of
-    Right j' -> regenerateScreens j' d ui{aopts=(aopts ui){cliopts_=copts}}
+    Right j' -> regenerateScreens j' d ui
     Left err ->
       case ui of
         UIState{aScreen=s@ErrorScreen{}} -> ui{aScreen=s{esError=err}}
@@ -172,8 +182,8 @@ uiReloadJournalIfChanged copts d j ui = do
 -- fail, enter (or update) the error screen. Or if balance assertions
 -- are disabled, do nothing.
 uiCheckBalanceAssertions :: Day -> UIState -> UIState
-uiCheckBalanceAssertions d ui@UIState{aopts=UIOpts{cliopts_=copts}, ajournal=j}
-  | ignore_assertions_ $ inputopts_ copts = ui
+uiCheckBalanceAssertions d ui@UIState{ajournal=j}
+  | ui^.ignore_assertions = ui
   | otherwise =
     case journalCheckBalanceAssertions j of
       Nothing  -> ui
