@@ -1,7 +1,9 @@
 -- The account register screen, showing transactions in an account, like hledger-web's register.
 
-{-# LANGUAGE OverloadedStrings, FlexibleContexts, RecordWildCards #-}
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE FlexibleContexts  #-}
+{-# LANGUAGE OverloadedStrings #-}
+{-# LANGUAGE RecordWildCards   #-}
+{-# LANGUAGE NamedFieldPuns #-}
 
 module Hledger.UI.RegisterScreen
  (registerScreen
@@ -14,18 +16,13 @@ where
 import Control.Monad
 import Control.Monad.IO.Class (liftIO)
 import Data.List
-import Data.List.Split (splitOn)
-#if !(MIN_VERSION_base(4,11,0))
-import Data.Monoid ((<>))
-#endif
 import Data.Maybe
 import qualified Data.Text as T
 import Data.Time.Calendar
 import qualified Data.Vector as V
-import Graphics.Vty (Event(..),Key(..),Modifier(..))
+import Graphics.Vty (Event(..),Key(..),Modifier(..), Button (BLeft, BScrollDown, BScrollUp))
 import Brick
-import Brick.Widgets.List
-  (handleListEvent, list, listElementsL, listMoveDown, listMoveTo, listNameL, listSelectedElement, listSelectedL, renderList)
+import Brick.Widgets.List hiding (reverse)
 import Brick.Widgets.Edit
 import Lens.Micro.Platform
 import Safe
@@ -42,6 +39,7 @@ import Hledger.UI.UIUtils
 import Hledger.UI.Editor
 import Hledger.UI.TransactionScreen
 import Hledger.UI.ErrorScreen
+import Data.Vector ((!?))
 
 registerScreen :: Screen
 registerScreen = RegisterScreen{
@@ -59,28 +57,28 @@ rsSetAccount a forceinclusive scr@RegisterScreen{} =
 rsSetAccount _ _ scr = scr
 
 rsInit :: Day -> Bool -> UIState -> UIState
-rsInit d reset ui@UIState{aopts=_uopts@UIOpts{cliopts_=CliOpts{reportopts_=ropts}}, ajournal=j, aScreen=s@RegisterScreen{..}} =
+rsInit d reset ui@UIState{aopts=_uopts@UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec@ReportSpec{_rsReportOpts=ropts}}}, ajournal=j, aScreen=s@RegisterScreen{..}} =
   ui{aScreen=s{rsList=newitems'}}
   where
     -- gather arguments and queries
     -- XXX temp
     inclusive = tree_ ropts || rsForceInclusive
     thisacctq = Acct $ (if inclusive then accountNameToAccountRegex else accountNameToAccountOnlyRegex) rsAccount
-    ropts' = ropts{
-               depth_=Nothing
-              }
-    q = And [queryFromOpts d ropts', excludeforecastq (forecast_ ropts)]
-      where
-        -- Except in forecast mode, exclude future/forecast transactions.
-        excludeforecastq (Just _) = Any
-        excludeforecastq Nothing  =  -- not:date:tomorrow- not:tag:generated-transaction
-          And [
-             Not (Date $ DateSpan (Just $ addDays 1 d) Nothing)
-            ,Not generatedTransactionTag
-          ]
 
-    (_label,items) = accountTransactionsReport ropts' j q thisacctq
-    items' = (if empty_ ropts' then id else filter (not . mixedAmountLooksZero . fifth6)) $  -- without --empty, exclude no-change txns
+    -- adjust the report options and regenerate the ReportSpec, carefully as usual to avoid screwups (#1523)
+    ropts' = ropts {
+        -- ignore any depth limit, as in postingsReport; allows register's total to match accounts screen
+        depth_=Nothing
+      -- XXX aregister also has this, needed ?
+        -- always show historical balance
+      -- , balanceaccum_= Historical
+      }
+    wd = whichDate ropts'
+    rspec' = reportSpecSetFutureAndForecast d (forecast_ $ inputopts_ copts) .
+      either (error "rsInit: adjusting the query for register, should not have failed") id $ -- PARTIAL:
+      updateReportSpec ropts' rspec{_rsDay=d}
+    items = accountTransactionsReport rspec' j thisacctq
+    items' = (if empty_ ropts then id else filter (not . mixedAmountLooksZero . fifth6)) $  -- without --empty, exclude no-change txns
              reverse  -- most recent last
              items
 
@@ -88,25 +86,25 @@ rsInit d reset ui@UIState{aopts=_uopts@UIOpts{cliopts_=CliOpts{reportopts_=ropts
     displayitems = map displayitem items'
       where
         displayitem (t, _, _issplit, otheracctsstr, change, bal) =
-          RegisterScreenItem{rsItemDate          = showDate $ transactionRegisterDate q thisacctq t
+          RegisterScreenItem{rsItemDate          = showDate $ transactionRegisterDate wd (_rsQuery rspec') thisacctq t
                             ,rsItemStatus        = tstatus t
-                            ,rsItemDescription   = T.unpack $ tdescription t
-                            ,rsItemOtherAccounts = case splitOn ", " otheracctsstr of
-                                                     [s] -> s
-                                                     ss  -> intercalate ", " ss
+                            ,rsItemDescription   = tdescription t
+                            ,rsItemOtherAccounts = otheracctsstr
                                                      -- _   -> "<split>"  -- should do this if accounts field width < 30
-                            ,rsItemChangeAmount  = showMixedAmountElided False change
-                            ,rsItemBalanceAmount = showMixedAmountElided False bal
+                            ,rsItemChangeAmount  = showamt change
+                            ,rsItemBalanceAmount = showamt bal
                             ,rsItemTransaction   = t
                             }
-    -- blank items are added to allow more control of scroll position; we won't allow movement over these
-    blankitems = replicate 100  -- 100 ought to be enough for anyone
+            where showamt = showMixedAmountB oneLine{displayMaxWidth=Just 32}
+    -- blank items are added to allow more control of scroll position; we won't allow movement over these.
+    -- XXX Ugly. Changing to 0 helps when debugging.
+    blankitems = replicate 100  -- "100 ought to be enough for anyone"
           RegisterScreenItem{rsItemDate          = ""
                             ,rsItemStatus        = Unmarked
                             ,rsItemDescription   = ""
                             ,rsItemOtherAccounts = ""
-                            ,rsItemChangeAmount  = ""
-                            ,rsItemBalanceAmount = ""
+                            ,rsItemChangeAmount  = mempty
+                            ,rsItemBalanceAmount = mempty
                             ,rsItemTransaction   = nulltransaction
                             }
     -- build the List
@@ -133,12 +131,12 @@ rsInit d reset ui@UIState{aopts=_uopts@UIOpts{cliopts_=CliOpts{reportopts_=ropts
                 nearestidbydatethenid = third3 <$> (headMay $ sort
                   [(abs $ diffDays (tdate t) prevseld, abs (tindex t - prevselidx), tindex t) | t <- ts])
                 ts = map rsItemTransaction displayitems
-        endidx = length displayitems - 1
+        endidx = max 0 $ length displayitems - 1
 
 rsInit _ _ _ = error "init function called with wrong screen type, should not happen"  -- PARTIAL:
 
 rsDraw :: UIState -> [Widget Name]
-rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
+rsDraw UIState{aopts=_uopts@UIOpts{uoCliOpts=copts@CliOpts{reportspec_=rspec}}
               ,aScreen=RegisterScreen{..}
               ,aMode=mode
               } =
@@ -164,8 +162,8 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
         whitespacewidth = 10 -- inter-column whitespace, fixed width
         minnonamtcolswidth = datewidth + 1 + 2 + 2 -- date column plus at least 1 for status and 2 for desc and accts
         maxamtswidth = max 0 (totalwidth - minnonamtcolswidth - whitespacewidth)
-        maxchangewidthseen = maximum' $ map (strWidth . rsItemChangeAmount) displayitems
-        maxbalwidthseen = maximum' $ map (strWidth . rsItemBalanceAmount) displayitems
+        maxchangewidthseen = maximum' $ map (wbWidth . rsItemChangeAmount) displayitems
+        maxbalwidthseen = maximum' $ map (wbWidth . rsItemBalanceAmount) displayitems
         changewidthproportion = fromIntegral maxchangewidthseen / fromIntegral (maxchangewidthseen + maxbalwidthseen)
         maxchangewidth = round $ changewidthproportion * fromIntegral maxamtswidth
         maxbalwidth = maxamtswidth - maxchangewidth
@@ -192,7 +190,8 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
       render $ defaultLayout toplabel bottomlabel $ renderList (rsDrawItem colwidths) True rsList
 
       where
-        ishistorical = balancetype_ ropts == HistoricalBalance
+        ropts = _rsReportOpts rspec
+        ishistorical = balanceaccum_ ropts == Historical
         -- inclusive = tree_ ropts || rsForceInclusive
 
         toplabel =
@@ -201,7 +200,7 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
           <+> togglefilters
           <+> str " transactions"
           -- <+> str (if ishistorical then " historical total" else " period total")
-          <+> borderQueryStr (query_ ropts)
+          <+> borderQueryStr (T.unpack . T.unwords . map textQuoteIfNeeded $ querystring_ ropts)
           -- <+> str " and subs"
           <+> borderPeriodStr "in" (period_ ropts)
           <+> str " ("
@@ -209,7 +208,7 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
           <+> str "/"
           <+> total
           <+> str ")"
-          <+> (if ignore_assertions_ $ inputopts_ copts then withAttr ("border" <> "query") (str " ignoring balance assertions") else str "")
+          <+> (if ignore_assertions_ . balancingopts_ $ inputopts_ copts then withAttr ("border" <> "query") (str " ignoring balance assertions") else str "")
           where
             togglefilters =
               case concat [
@@ -223,13 +222,13 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
                          Nothing -> "-"
                          Just i -> show (i + 1)
             total = str $ show $ length nonblanks
-            nonblanks = V.takeWhile (not . null . rsItemDate) $ rsList^.listElementsL
+            nonblanks = V.takeWhile (not . T.null . rsItemDate) $ rsList^.listElementsL
 
             -- query = query_ $ reportopts_ $ cliopts_ opts
 
         bottomlabel = case mode of
-                        Minibuffer ed -> minibuffer ed
-                        _             -> quickhelp
+                        Minibuffer label ed -> minibuffer label ed
+                        _                   -> quickhelp
           where
             quickhelp = borderKeysStr' [
                ("?", str "help")
@@ -242,7 +241,7 @@ rsDraw UIState{aopts=_uopts@UIOpts{cliopts_=copts@CliOpts{reportopts_=ropts}}
               -- ,("l", str "list(-subs)")
 
               ,("H", renderToggle (not ishistorical) "historical" "period")
-              ,("F", renderToggle1 (isJust $ forecast_ ropts) "forecast")
+              ,("F", renderToggle1 (isJust . forecast_ . inputopts_ $ copts) "forecast")
 --               ,("a", "add")
 --               ,("g", "reload")
 --               ,("q", "quit")
@@ -254,51 +253,57 @@ rsDrawItem :: (Int,Int,Int,Int,Int) -> Bool -> RegisterScreenItem -> Widget Name
 rsDrawItem (datewidth,descwidth,acctswidth,changewidth,balwidth) selected RegisterScreenItem{..} =
   Widget Greedy Fixed $ do
     render $
-      str (fitString (Just datewidth) (Just datewidth) True True rsItemDate) <+>
-      str " " <+>
-      str (fitString (Just 1) (Just 1) True True (show rsItemStatus)) <+>
-      str " " <+>
-      str (fitString (Just descwidth) (Just descwidth) True True rsItemDescription) <+>
-      str "  " <+>
-      str (fitString (Just acctswidth) (Just acctswidth) True True rsItemOtherAccounts) <+>
-      str "   " <+>
-      withAttr changeattr (str (fitString (Just changewidth) (Just changewidth) True False rsItemChangeAmount)) <+>
-      str "   " <+>
-      withAttr balattr (str (fitString (Just balwidth) (Just balwidth) True False rsItemBalanceAmount))
+      txt (fitText (Just datewidth) (Just datewidth) True True rsItemDate) <+>
+      txt " " <+>
+      txt (fitText (Just 1) (Just 1) True True (T.pack $ show rsItemStatus)) <+>
+      txt " " <+>
+      txt (fitText (Just descwidth) (Just descwidth) True True rsItemDescription) <+>
+      txt "  " <+>
+      txt (fitText (Just acctswidth) (Just acctswidth) True True rsItemOtherAccounts) <+>
+      txt "   " <+>
+      withAttr changeattr (txt $ fitText (Just changewidth) (Just changewidth) True False changeAmt) <+>
+      txt "   " <+>
+      withAttr balattr (txt $ fitText (Just balwidth) (Just balwidth) True False balanceAmt)
   where
-    changeattr | '-' `elem` rsItemChangeAmount = sel $ "list" <> "amount" <> "decrease"
-               | otherwise                     = sel $ "list" <> "amount" <> "increase"
-    balattr    | '-' `elem` rsItemBalanceAmount = sel $ "list" <> "balance" <> "negative"
-               | otherwise                      = sel $ "list" <> "balance" <> "positive"
+    changeAmt  = wbToText rsItemChangeAmount
+    balanceAmt = wbToText rsItemBalanceAmount
+    changeattr | T.any (=='-') changeAmt  = sel $ "list" <> "amount" <> "decrease"
+               | otherwise                = sel $ "list" <> "amount" <> "increase"
+    balattr    | T.any (=='-') balanceAmt = sel $ "list" <> "balance" <> "negative"
+               | otherwise                = sel $ "list" <> "balance" <> "positive"
     sel | selected  = (<> "selected")
         | otherwise = id
 
 rsHandle :: UIState -> BrickEvent Name AppEvent -> EventM Name (Next UIState)
 rsHandle ui@UIState{
    aScreen=s@RegisterScreen{..}
-  ,aopts=UIOpts{cliopts_=copts}
+  ,aopts=UIOpts{uoCliOpts=copts}
   ,ajournal=j
   ,aMode=mode
   } ev = do
-  d <- liftIO getCurrentDay
   let
+    d = copts^.rsDay
     journalspan = journalDateSpan False j
-    nonblanks = V.takeWhile (not . null . rsItemDate) $ rsList^.listElementsL
+    nonblanks = V.takeWhile (not . T.null . rsItemDate) $ rsList^.listElementsL
     lastnonblankidx = max 0 (length nonblanks - 1)
 
   case mode of
-    Minibuffer ed ->
+    Minibuffer _ ed ->
       case ev of
         VtyEvent (EvKey KEsc   []) -> continue $ closeMinibuffer ui
-        VtyEvent (EvKey KEnter []) -> continue $ regenerateScreens j d $ setFilter s $ closeMinibuffer ui
-                            where s = chomp $ unlines $ map strip $ getEditContents ed
+        VtyEvent (EvKey KEnter []) -> continue $ regenerateScreens j d $
+            case setFilter s $ closeMinibuffer ui of
+              Left bad -> showMinibuffer "Cannot compile regular expression" (Just bad) ui
+              Right ui' -> ui'
+          where s = chomp . unlines . map strip $ getEditContents ed
+        -- VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer ui
         VtyEvent (EvKey (KChar 'l') [MCtrl]) -> redraw ui
         VtyEvent (EvKey (KChar 'z') [MCtrl]) -> suspend ui
         VtyEvent ev              -> do ed' <- handleEditorEvent ev ed
-                                       continue $ ui{aMode=Minibuffer ed'}
+                                       continue $ ui{aMode=Minibuffer "filter" ed'}
         AppEvent _        -> continue ui
-        MouseDown _ _ _ _ -> continue ui
-        MouseUp _ _ _     -> continue ui
+        MouseDown{}       -> continue ui
+        MouseUp{}         -> continue ui
 
     Help ->
       case ev of
@@ -311,7 +316,7 @@ rsHandle ui@UIState{
       case ev of
         VtyEvent (EvKey (KChar 'q') []) -> halt ui
         VtyEvent (EvKey KEsc        []) -> continue $ resetScreens d ui
-        VtyEvent (EvKey (KChar c)   []) | c `elem` ['?'] -> continue $ setMode Help ui
+        VtyEvent (EvKey (KChar c)   []) | c == '?' -> continue $ setMode Help ui
         AppEvent (DateChange old _) | isStandardPeriod p && p `periodContainsDate` old ->
           continue $ regenerateScreens j d $ setReportPeriod (DayPeriod d) ui
           where
@@ -327,53 +332,64 @@ rsHandle ui@UIState{
             (pos,f) = case listSelectedElement rsList of
                         Nothing -> (endPosition, journalFilePath j)
                         Just (_, RegisterScreenItem{
-                          rsItemTransaction=Transaction{tsourcepos=GenericSourcePos f l c}}) -> (Just (l, Just c),f)
-                        Just (_, RegisterScreenItem{
-                          rsItemTransaction=Transaction{tsourcepos=JournalSourcePos f (l,_)}}) -> (Just (l, Nothing),f)
+                          rsItemTransaction=Transaction{tsourcepos=(SourcePos f l c,_)}}) -> (Just (unPos l, Just $ unPos c),f)
 
         -- display mode/query toggles
-        VtyEvent (EvKey (KChar 'B') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleCost ui
+        VtyEvent (EvKey (KChar 'B') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleConversionOp ui
         VtyEvent (EvKey (KChar 'V') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleValue ui
         VtyEvent (EvKey (KChar 'H') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleHistorical ui
         VtyEvent (EvKey (KChar 't') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleTree ui
-        VtyEvent (EvKey (KChar 'Z') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleEmpty ui
+        VtyEvent (EvKey (KChar c) []) | c `elem` ['z','Z'] -> rsCenterAndContinue $ regenerateScreens j d $ toggleEmpty ui
         VtyEvent (EvKey (KChar 'R') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleReal ui
         VtyEvent (EvKey (KChar 'U') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleUnmarked ui
         VtyEvent (EvKey (KChar 'P') []) -> rsCenterAndContinue $ regenerateScreens j d $ togglePending ui
         VtyEvent (EvKey (KChar 'C') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleCleared ui
         VtyEvent (EvKey (KChar 'F') []) -> rsCenterAndContinue $ regenerateScreens j d $ toggleForecast d ui
 
-        VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer ui
+        VtyEvent (EvKey (KChar '/') []) -> continue $ regenerateScreens j d $ showMinibuffer "filter" Nothing ui
         VtyEvent (EvKey (KDown)     [MShift]) -> continue $ regenerateScreens j d $ shrinkReportPeriod d ui
         VtyEvent (EvKey (KUp)       [MShift]) -> continue $ regenerateScreens j d $ growReportPeriod d ui
         VtyEvent (EvKey (KRight)    [MShift]) -> continue $ regenerateScreens j d $ nextReportPeriod journalspan ui
         VtyEvent (EvKey (KLeft)     [MShift]) -> continue $ regenerateScreens j d $ previousReportPeriod journalspan ui
         VtyEvent (EvKey k           []) | k `elem` [KBS, KDel] -> (continue $ regenerateScreens j d $ resetFilter ui)
-        VtyEvent e | e `elem` moveLeftEvents  -> continue $ popScreen ui
         VtyEvent (EvKey (KChar 'l') [MCtrl]) -> scrollSelectionToMiddle rsList >> redraw ui
         VtyEvent (EvKey (KChar 'z') [MCtrl]) -> suspend ui
 
-        -- enter transaction screen for selected transaction
-        VtyEvent e | e `elem` moveRightEvents -> do
-          case listSelectedElement rsList of
-            Just (_, RegisterScreenItem{rsItemTransaction=t}) ->
-              let
-                ts = map rsItemTransaction $ V.toList $ nonblanks
-                numberedts = zip [1..] ts
-                i = maybe 0 (toInteger . (+1)) $ elemIndex t ts -- XXX
-              in
-                continue $ screenEnter d transactionScreen{tsTransaction=(i,t)
-                                                          ,tsTransactions=numberedts
-                                                          ,tsAccount=rsAccount} ui
-            Nothing -> continue ui
+        -- exit screen on LEFT
+        VtyEvent e | e `elem` moveLeftEvents  -> continue $ popScreen ui
+        -- or on a click in the app's left margin. This is a VtyEvent since not in a clickable widget.
+        VtyEvent (EvMouseUp x _y (Just BLeft)) | x==0 -> continue $ popScreen ui
+        -- or on clicking a blank list item.
+        MouseUp _ (Just BLeft) Location{loc=(_,y)} | clickeddate == "" -> continue $ popScreen ui
+          where clickeddate = maybe "" rsItemDate $ listElements rsList !? y
 
-        -- prevent moving down over blank padding items;
-        -- instead scroll down by one, until maximally scrolled - shows the end has been reached
+        -- enter transaction screen on RIGHT
+        VtyEvent e | e `elem` moveRightEvents ->
+          case listSelectedElement rsList of
+            Just _  -> continue $ screenEnter d transactionScreen{tsAccount=rsAccount} ui
+            Nothing -> continue ui
+        -- or on transaction click
+        -- MouseDown is sometimes duplicated, https://github.com/jtdaugherty/brick/issues/347
+        -- just use it to move the selection
+        MouseDown _n BLeft _mods Location{loc=(_x,y)} | not $ (=="") clickeddate -> do
+          continue $ ui{aScreen=s{rsList=listMoveTo y rsList}}
+          where clickeddate = maybe "" rsItemDate $ listElements rsList !? y
+        -- and on MouseUp, enter the subscreen
+        MouseUp _n (Just BLeft) Location{loc=(_x,y)} | not $ (=="") clickeddate -> do
+          continue $ screenEnter d transactionScreen{tsAccount=rsAccount} ui
+          where clickeddate = maybe "" rsItemDate $ listElements rsList !? y
+
+        -- when selection is at the last item, DOWN scrolls instead of moving, until maximally scrolled
         VtyEvent e | e `elem` moveDownEvents, isBlankElement mnextelement -> do
-          vScrollBy (viewportScroll $ rsList^.listNameL) 1
-          continue ui
-          where
-            mnextelement = listSelectedElement $ listMoveDown rsList
+          vScrollBy (viewportScroll $ rsList ^. listNameL) 1 >> continue ui
+          where mnextelement = listSelectedElement $ listMoveDown rsList
+
+        -- mouse scroll wheel scrolls the viewport up or down to its maximum extent,
+        -- pushing the selection when necessary.
+        MouseDown name btn _mods _loc | btn `elem` [BScrollUp, BScrollDown] -> do
+          let scrollamt = if btn==BScrollUp then -1 else 1
+          list' <- listScrollPushingSelection name rsList (rsListSize rsList) scrollamt
+          continue ui{aScreen=s{rsList=list'}}
 
         -- if page down or end leads to a blank padding item, stop at last non-blank
         VtyEvent e@(EvKey k           []) | k `elem` [KPageDown, KEnd] -> do
@@ -392,9 +408,9 @@ rsHandle ui@UIState{
           newitems <- handleListEvent ev' rsList
           continue ui{aScreen=s{rsList=newitems}}
 
+        MouseDown{}       -> continue ui
+        MouseUp{}         -> continue ui
         AppEvent _        -> continue ui
-        MouseDown _ _ _ _ -> continue ui
-        MouseUp _ _ _     -> continue ui
 
 rsHandle _ _ = error "event handler called with wrong screen type, should not happen"  -- PARTIAL:
 
@@ -403,3 +419,5 @@ isBlankElement mel = ((rsItemDate . snd) <$> mel) == Just ""
 rsCenterAndContinue ui = do
   scrollSelectionToMiddle $ rsList $ aScreen ui
   continue ui
+
+rsListSize = V.length . V.takeWhile ((/="").rsItemDate) . listElements
