@@ -1,31 +1,40 @@
 {-# LANGUAGE CPP #-}
-{-# LANGUAGE LambdaCase #-}
 {-|
 
 -}
 
 module Hledger.UI.UIOptions
 where
-import Data.Default
+
+import Data.Default (def)
 import Data.List (intercalate)
-import System.Environment
+import qualified Data.Map as M
+import Data.Maybe (fromMaybe)
+import Lens.Micro (set)
+import System.Environment (getArgs)
 
-import Hledger.Cli hiding (progname,version,prognameandversion)
-import Hledger.UI.Theme (themeNames)
+import Hledger.Cli hiding (packageversion, progname, prognameandversion)
+import Hledger.UI.Theme (themes, themeNames)
 
-progname, version :: String
-progname = "hledger-ui"
+-- cf Hledger.Cli.Version
+
+packageversion :: PackageVersion
+packageversion =
 #ifdef VERSION
-version = VERSION
+  VERSION
 #else
-version = ""
+  ""
 #endif
-prognameandversion :: String
-prognameandversion = progname ++ " " ++ version :: String
+
+progname :: ProgramName
+progname = "hledger-ui"
+
+prognameandversion :: VersionString
+prognameandversion = versionString progname packageversion
 
 uiflags = [
   -- flagNone ["debug-ui"] (setboolopt "rules-file") "run with no terminal output, showing console"
-   flagNone ["watch"] (setboolopt "watch") "watch for data and date changes and reload automatically"
+   flagNone ["watch","w"] (setboolopt "watch") "watch for data and date changes and reload automatically"
   ,flagReq  ["theme"] (\s opts -> Right $ setopt "theme" s opts) "THEME" ("use this custom display theme ("++intercalate ", " themeNames++")")
   ,flagReq  ["register"] (\s opts -> Right $ setopt "register" s opts) "ACCTREGEX" "start in the (first) matched account's register"
   ,flagNone ["change"] (setboolopt "change")
@@ -57,34 +66,35 @@ uimode =  (mode "hledger-ui" (setopt "command" "ui" def)
            }
 
 -- hledger-ui options, used in hledger-ui and above
-data UIOpts = UIOpts {
-     watch_   :: Bool
-    ,change_  :: Bool
-    ,cliopts_ :: CliOpts
- } deriving (Show)
+data UIOpts = UIOpts
+  { uoWatch    :: Bool
+  , uoTheme    :: Maybe String
+  , uoRegister :: Maybe String
+  , uoCliOpts  :: CliOpts
+  } deriving (Show)
 
 defuiopts = UIOpts
-    def
-    def
-    def
-
--- instance Default CliOpts where def = defcliopts
+  { uoWatch    = False
+  , uoTheme    = Nothing
+  , uoRegister = Nothing
+  , uoCliOpts  = defcliopts
+  }
 
+-- | Process a RawOpts into a UIOpts.
+-- This will return a usage error if provided an invalid theme.
 rawOptsToUIOpts :: RawOpts -> IO UIOpts
-rawOptsToUIOpts rawopts = checkUIOpts <$> do
-  cliopts <- rawOptsToCliOpts rawopts
-  return defuiopts {
-              watch_   = boolopt "watch" rawopts
-             ,change_  = boolopt "change" rawopts
-             ,cliopts_ = cliopts
-             }
-
-checkUIOpts :: UIOpts -> UIOpts
-checkUIOpts opts =
-  either usageError (const opts) $ do
-    case maybestringopt "theme" $ rawopts_ $ cliopts_ opts of
-      Just t | not $ elem t themeNames -> Left $ "invalid theme name: "++t
-      _                                -> Right ()
+rawOptsToUIOpts rawopts = do
+    cliopts <- set balanceaccum accum <$> rawOptsToCliOpts rawopts
+    return defuiopts {
+                uoWatch    = boolopt "watch" rawopts
+               ,uoTheme    = checkTheme <$> maybestringopt "theme" rawopts
+               ,uoRegister = maybestringopt "register" rawopts
+               ,uoCliOpts  = cliopts
+               }
+  where
+    -- show historical balance by default (unlike hledger)
+    accum = fromMaybe Historical $ balanceAccumulationOverride rawopts
+    checkTheme t = if t `M.member` themes then t else usageError $ "invalid theme name: " ++ t
 
 -- XXX some refactoring seems due
 getHledgerUIOpts :: IO UIOpts
@@ -95,3 +105,20 @@ getHledgerUIOpts = do
   let cmdargopts = either usageError id $ process uimode args'
   rawOptsToUIOpts cmdargopts
 
+instance HasCliOpts UIOpts where
+    cliOpts f uiopts = (\x -> uiopts{uoCliOpts=x}) <$> f (uoCliOpts uiopts)
+
+instance HasInputOpts UIOpts where
+    inputOpts = cliOpts.inputOpts
+
+instance HasBalancingOpts UIOpts where
+    balancingOpts = cliOpts.balancingOpts
+
+instance HasReportSpec UIOpts where
+    reportSpec = cliOpts.reportSpec
+
+instance HasReportOptsNoUpdate UIOpts where
+    reportOptsNoUpdate = cliOpts.reportOptsNoUpdate
+
+instance HasReportOpts UIOpts where
+    reportOpts = cliOpts.reportOpts
