@@ -38,6 +38,7 @@ Brick.defaultMain brickapp st
 
 module Hledger.UI.UITypes where
 
+import Data.Text (Text)
 import Data.Time.Calendar (Day)
 import Brick
 import Brick.Widgets.List (List)
@@ -47,6 +48,7 @@ import Text.Show.Functions ()
   -- import the Show instance for functions. Warning, this also re-exports it
 
 import Hledger
+import Hledger.Cli (HasCliOpts(..))
 import Hledger.UI.UIOptions
 
 -- | hledger-ui's application state. This holds one or more stateful screens.
@@ -67,7 +69,7 @@ data UIState = UIState {
 data Mode =
     Normal
   | Help
-  | Minibuffer (Editor String Name)
+  | Minibuffer Text (Editor String Name)
   deriving (Show,Eq)
 
 -- Ignore the editor when comparing Modes.
@@ -81,6 +83,7 @@ data Name =
   | AccountsList
   | RegisterViewport
   | RegisterList
+  | TransactionEditor
   deriving (Ord, Show, Eq)
 
 data AppEvent =
@@ -133,22 +136,21 @@ data Screen =
 
 -- | An item in the accounts screen's list of accounts and balances.
 data AccountsScreenItem = AccountsScreenItem {
-   asItemIndentLevel        :: Int          -- ^ indent level
-  ,asItemAccountName        :: AccountName  -- ^ full account name
-  ,asItemDisplayAccountName :: AccountName  -- ^ full or short account name to display
-  ,asItemRenderedAmounts    :: [String]     -- ^ rendered amounts
-  }
-  deriving (Show)
+   asItemIndentLevel        :: Int                -- ^ indent level
+  ,asItemAccountName        :: AccountName        -- ^ full account name
+  ,asItemDisplayAccountName :: AccountName        -- ^ full or short account name to display
+  ,asItemMixedAmount        :: Maybe MixedAmount  -- ^ mixed amount to display
+  } deriving (Show)
 
 -- | An item in the register screen's list of transactions in the current account.
 data RegisterScreenItem = RegisterScreenItem {
-   rsItemDate           :: String           -- ^ date
-  ,rsItemStatus         :: Status           -- ^ transaction status
-  ,rsItemDescription    :: String           -- ^ description
-  ,rsItemOtherAccounts  :: String           -- ^ other accounts
-  ,rsItemChangeAmount   :: String           -- ^ the change to the current account from this transaction
-  ,rsItemBalanceAmount  :: String           -- ^ the balance or running total after this transaction
-  ,rsItemTransaction    :: Transaction      -- ^ the full transaction
+   rsItemDate           :: Text         -- ^ date
+  ,rsItemStatus         :: Status       -- ^ transaction status
+  ,rsItemDescription    :: Text         -- ^ description
+  ,rsItemOtherAccounts  :: Text         -- ^ other accounts
+  ,rsItemChangeAmount   :: WideBuilder  -- ^ the change to the current account from this transaction
+  ,rsItemBalanceAmount  :: WideBuilder  -- ^ the balance or running total after this transaction
+  ,rsItemTransaction    :: Transaction  -- ^ the full transaction
   }
   deriving (Show)
 
@@ -160,7 +162,24 @@ type NumberedTransaction = (Integer, Transaction)
 --    mempty        = list "" V.empty 1  -- XXX problem in 0.7, every list requires a unique Name
 --    mappend l1 l2 = l1 & listElementsL .~ (l1^.listElementsL <> l2^.listElementsL)
 
-concat <$> mapM makeLenses [
-   ''Screen
-  ]
+makeLenses ''Screen
 
+uioptslens f ui = (\x -> ui{aopts=x}) <$> f (aopts ui)
+
+instance HasCliOpts UIState where
+    cliOpts = uioptslens.cliOpts
+
+instance HasInputOpts UIState where
+    inputOpts = uioptslens.inputOpts
+
+instance HasBalancingOpts UIState where
+    balancingOpts = uioptslens.balancingOpts
+
+instance HasReportSpec UIState where
+    reportSpec = uioptslens.reportSpec
+
+instance HasReportOptsNoUpdate UIState where
+    reportOptsNoUpdate = uioptslens.reportOptsNoUpdate
+
+instance HasReportOpts UIState where
+    reportOpts = uioptslens.reportOpts
